/*
Copyright (C) 2001 StrmnNrmn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// Display stuff like registers, instructions, memory usage and so on
#include "stdafx.h"

#include <shlwapi.h>

#include "ultra_r4300.h"

#include "Memory.h"
#include "CPU.h"
#include "RSP.h"
#include "Interrupt.h"
#include "Dump.h"
#include "OpCode.h"
#include "debug.h"
#include "patch.h"		// For GetCorrectOp


static TCHAR g_szDumpDir[MAX_PATH+1] = TEXT("");

// Initialise the directory where files are dumped
void Dump_GetDumpDirectory(LPTSTR szFilePath, LPCTSTR szSubDir)
{
	// Appends szBase to the global dump base. Stores in szFilePath

	if (lstrlen(g_szDumpDir) == 0)
	{
		// Initialise
		PathCombine(g_szDumpDir, g_szDaedalusExeDir, TEXT("Dumps"));

		// Create if it doesn't already exist
		if (!PathIsDirectory(g_szDumpDir))
		{
			CreateDirectory(g_szDumpDir, NULL);
		}
	}

	// If a subdirectory was specified, append 
	if (lstrlen(szSubDir) > 0)
	{
		PathCombine(szFilePath, g_szDumpDir, szSubDir);
		if (!PathIsDirectory(szFilePath))
		{
			CreateDirectory(szFilePath, NULL);
		}
	}

	else
	{
		lstrcpyn(szFilePath, g_szDumpDir, MAX_PATH);

		// Directory already exists
	}

}




// pdwBase[0] = instruction at dwStart
static void DumpRange(LPCTSTR szFileName,
					  LPCTSTR szMode,
					  DWORD * pdwBase,
					  DWORD dwStart,
					  DWORD dwEnd,
					  BOOL bTranslatePatches)
{
	char opinfo[400];
	FILE * fp;	

	fp = fopen(szFileName, szMode);
	if (fp == NULL)
		return;

	DBGConsole_Msg(0, "Dumping %s", szFileName);

	for (DWORD dwPC = dwStart; dwPC < dwEnd; dwPC+=4)
	{
		DWORD dwOriginalOp = pdwBase[(dwPC-dwStart)/4];
		DWORD dwInstrNum = (dwPC&0x00FFFFFF)>>2;
		DWORD dwOpCode;

		if (bTranslatePatches)
		{
			dwOpCode = GetCorrectOp(dwInstrNum, dwOriginalOp);
			if (IsJumpTarget(dwInstrNum, dwOriginalOp))
			{
				fprintf(fp, "\n");
				fprintf(fp, "\n");
				fprintf(fp, "// %s():\n", Patch_GetJumpAddressName(dwPC));
			}
		}			
		else
		{
			dwOpCode = GetCorrectOp(dwInstrNum, dwOriginalOp);
		}

		SprintOpCodeInfo(opinfo, dwPC, dwOpCode);
		fprintf(fp, "0x%08x: <0x%08x> %s\n", dwPC, dwOpCode, opinfo);
	}

	fclose(fp);
}

void Dump_OSRange(LPCTSTR szFileName)
{
	//DWORD dwStart = 0x80008000;
	//DWORD dwEnd   = 0x80240000;
	//0x00236190 -> 0x002454d0
	static const DWORD dwStart = 0x80000000;
	static const DWORD dwEnd   = 0x80600000;
	TCHAR szFilePath[MAX_PATH+1];

	if (szFileName == NULL || lstrlen(szFileName) == 0)
	{
		Dump_GetDumpDirectory(szFilePath, TEXT(""));
		PathAppend(szFilePath, TEXT("os_range.txt"));
	}
	else
	{
		lstrcpyn(szFilePath, szFileName, MAX_PATH);
	}

	
	DWORD * pdwBase;
	if (InternalReadAddress(dwStart, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", dwStart);
	}
	else
	{
		//DumpRange(0x80300000,0x80380000);
		//DumpRange(0x80240000,0x80300000);
		DumpRange(szFilePath, "w", pdwBase, dwStart, dwEnd, TRUE);
	}
}

void Dump_BootROMRange(LPCTSTR szFileName)
{
	static const DWORD dwStart = 0xB0000000;
	static const DWORD dwEnd   = 0xB0001000;
	TCHAR szFilePath[MAX_PATH+1];

	if (szFileName == NULL || lstrlen(szFileName) == 0)
	{
		Dump_GetDumpDirectory(szFilePath, TEXT(""));
		PathAppend(szFilePath, TEXT("boot_rom.txt"));
	}
	else
	{
		lstrcpyn(szFilePath, szFileName, MAX_PATH);
	}
	
	
	DWORD * pdwBase;
	if (InternalReadAddress(dwStart, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", dwStart);
	}
	else
	{
		DumpRange(szFilePath, "w", pdwBase, 0xa4000000, 0xa4000000+0x1000, FALSE);// dwStart, dwEnd, FALSE);
	}
}

void  Dump_ExceptionVectors(LPCTSTR szFileName)
{
	DWORD * pdwBase;
	TCHAR szFilePath[MAX_PATH+1];

	if (szFileName == NULL || lstrlen(szFileName) == 0)
	{
		Dump_GetDumpDirectory(szFilePath, TEXT(""));
		PathAppend(szFilePath, TEXT("exc_vectors.txt"));
	}
	else
	{
		lstrcpyn(szFilePath, szFileName, MAX_PATH);
	}


	// TLB
	if (InternalReadAddress(UT_VEC, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", UT_VEC);
	}
	else
	{
		DumpRange(szFilePath, "w", pdwBase, UT_VEC,UT_VEC+SIZE_EXCVEC, TRUE);
	}

	// Refilll
	if (InternalReadAddress(R_VEC, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", R_VEC);
	}
	else
	{
		DumpRange(szFilePath, "a", pdwBase, R_VEC,R_VEC+SIZE_EXCVEC, TRUE);
	}

	// Xtended TLB
	if (InternalReadAddress(XUT_VEC, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", XUT_VEC);
	}
	else
	{
		DumpRange(szFilePath, "a", pdwBase, XUT_VEC,XUT_VEC+SIZE_EXCVEC, TRUE);
	}


	// ECC
	if (InternalReadAddress(ECC_VEC, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", ECC_VEC);
	}
	else
	{
		DumpRange(szFilePath, "a", pdwBase, ECC_VEC,ECC_VEC+SIZE_EXCVEC, TRUE);
	}

	// General
	if (InternalReadAddress(E_VEC, (void**)&pdwBase) == MEM_UNUSED)
	{
		DBGConsole_Msg(0, "Invalid base 0x%08x", E_VEC);
	}
	else
	{
		DumpRange(szFilePath, "a", pdwBase, E_VEC,E_VEC+SIZE_EXCVEC, TRUE);
	}
	
}



