/*
Copyright (C) 2001 StrmnNrmn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "stdafx.h"
#include <shlwapi.h>
#include <windowsx.h>		// ComboBox_ defines
#include <commctrl.h>		// LVITEM etc
#include "resource.h"
#include "InputDialog.h"
#include "DBGConsole.h"
#include "ConfigHandler.h"
#include "DInputHandler.h"

static BOOL CALLBACK InputDialogProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static BOOL InputDialog_OnInitDialog(HWND hWndDlg, WPARAM wParam, LPARAM lParam);

static void InputDialog_FillControllerCombo(HWND hWndDlg);
static void InputDialog_FillButtonList(HWND hWndDlg);
static void InputDialog_FillMapCombo(HWND hWndDlg);

static void InputDialog_AssignA(DWORD iButton, DWORD iControl);
static void InputDialog_AssignB(DWORD iButton, DWORD iControl);

static void InputDialog_SelectMapComboEntry(HWND hWndDl, DWORD dwID);

static ControlList s_CL;
static ButtonAssignment s_Buttons[NUM_INPUT_IDS];

void InputDialog_DoModal(HWND hWndParent)
{
	DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_INPUT), hWndParent, InputDialogProc);

	s_CL.controls.clear();
}

#define CheckBox_GetCheck(hwnd)			SendMessage(hwnd, BM_GETCHECK, 0,0)
#define CheckBox_SetCheck(hwnd, state)	SendMessage(hwnd, BM_SETCHECK, (WPARAM)state,0)


BOOL CALLBACK InputDialogProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG iButton;
	LONG iControl;
	LONG nSelectedButton;
	LONG nSelectedControl;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		return InputDialog_OnInitDialog(hWndDlg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_RESET_BUTTON:
			{
				Input_RescanDevices();
				memcpy(s_Buttons, g_CurrentButtons, sizeof(s_Buttons));

				// Refresh
				nSelectedButton = ListBox_GetCurSel(GetDlgItem(hWndDlg, IDC_BUTTON_LIST));
				if (nSelectedButton != LB_ERR)
				{
					iButton = ListBox_GetItemData(GetDlgItem(hWndDlg, IDC_BUTTON_LIST), nSelectedButton);
					InputDialog_SelectMapComboEntry(hWndDlg, iButton);
				}

			}
			break;
		case IDC_BUTTON_LIST:
			if (HIWORD(wParam) == LBN_SELCHANGE)
			{
				nSelectedButton = ListBox_GetCurSel(GetDlgItem(hWndDlg, IDC_BUTTON_LIST));
				if (nSelectedButton != LB_ERR)
				{
					iButton = ListBox_GetItemData(GetDlgItem(hWndDlg, IDC_BUTTON_LIST), nSelectedButton);
					InputDialog_SelectMapComboEntry(hWndDlg, iButton);
				}
			}
			break;
		case IDC_FIRST_MAP_COMBO:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				nSelectedControl = ComboBox_GetCurSel(GetDlgItem(hWndDlg, IDC_FIRST_MAP_COMBO));
				nSelectedButton = ListBox_GetCurSel(GetDlgItem(hWndDlg, IDC_BUTTON_LIST));

				if (nSelectedControl != CB_ERR && nSelectedButton != LB_ERR)
				{
					iControl = ComboBox_GetItemData(GetDlgItem(hWndDlg, IDC_FIRST_MAP_COMBO), nSelectedControl);
					iButton = ListBox_GetItemData(GetDlgItem(hWndDlg, IDC_BUTTON_LIST), nSelectedButton);

					// iControl is index into s_CL.controls
					InputDialog_AssignA(iButton, iControl);

					// Refresh
					InputDialog_SelectMapComboEntry(hWndDlg, iButton);
				}
			}
			break;

		case IDC_SECOND_MAP_COMBO:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				nSelectedControl = ComboBox_GetCurSel(GetDlgItem(hWndDlg, IDC_SECOND_MAP_COMBO));
				nSelectedButton = ListBox_GetCurSel(GetDlgItem(hWndDlg, IDC_BUTTON_LIST));

				if (nSelectedControl != CB_ERR && nSelectedButton != LB_ERR)
				{
					iControl = ComboBox_GetItemData(GetDlgItem(hWndDlg, IDC_SECOND_MAP_COMBO), nSelectedControl);
					iButton = ListBox_GetItemData(GetDlgItem(hWndDlg, IDC_BUTTON_LIST), nSelectedButton);

					// iControl is index into s_CL.controls
					InputDialog_AssignB(iButton, iControl);

					// Refresh
					//InputDialog_SelectMapComboEntry(hWndDlg, iButton);
				}
			}
			break;

		case IDOK:
			// Copy our copy of the data back, setting up the new values
			memcpy(g_CurrentButtons, s_Buttons, sizeof(s_Buttons));
			EndDialog(hWndDlg, IDOK);
			return TRUE;
		case IDCANCEL:
			EndDialog(hWndDlg, IDCANCEL);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		// Free any plugins that were found
		return TRUE;

	}


	return FALSE;
}

BOOL InputDialog_OnInitDialog(HWND hWndDlg, WPARAM wParam, LPARAM lParam)
{
	// Make a copy of the data
	// If the user hits cancel, this information is discarded.
	// If they hit ok, this information is copied back across
	memcpy(s_Buttons, g_CurrentButtons, sizeof(s_Buttons));

	// Initialise the lists
	InputDialog_FillControllerCombo(hWndDlg);
	InputDialog_FillMapCombo(hWndDlg);			// Have to do this before button list is filled
	InputDialog_FillButtonList(hWndDlg);


	SetFocus(GetDlgItem(hWndDlg, IDC_CONTROLLER_COMBO));

	// We set the focus, return false
	return FALSE;
}


void InputDialog_FillControllerCombo(HWND hWndDlg)
{

	LONG i;
	HWND hWndCombo;
	CHAR szName[100];
	LONG nIndex;
	LONG nSelection;

	hWndCombo = GetDlgItem(hWndDlg, IDC_CONTROLLER_COMBO);

	ComboBox_ResetContent(hWndCombo);

	nSelection = -1;

	for (i = 0; i < 4; i++)
	{
		wsprintf(szName, "Controller %d", i);

		nIndex = ComboBox_InsertString(hWndCombo, -1, szName);

		if (nIndex != CB_ERR && nIndex != CB_ERRSPACE)
		{
			// Controller 1 is the initial selection
			if (i == 0)
			{
				nSelection = nIndex;
			}

			// Set item data
			ComboBox_SetItemData(hWndCombo, nIndex, i);
		}
	}

	if (nSelection == -1)
		nSelection = 0;

	ComboBox_SetCurSel(hWndCombo, nSelection);

}

void InputDialog_FillButtonList(HWND hWndDlg)
{

	LONG i;
	HWND hWndList;
	LONG nSelection;
	LONG nIndex;

	hWndList = GetDlgItem(hWndDlg, IDC_BUTTON_LIST);

	ListBox_ResetContent(hWndList);

	// Set selected item to first in list
	nSelection = -1;

	for (i = 0; i < NUM_INPUT_IDS; i++)
	{
		nIndex = ListBox_AddString(hWndList, g_N64Buttons[i].szName);
		if (nIndex != LB_ERR && nIndex != LB_ERRSPACE)
		{
			if (i == 0)
			{
				nSelection = nIndex;
			}

			// Set item data
			ListBox_SetItemData(hWndList, nIndex, i);	// The item data is the ID value

		}
	}
	if (nSelection == -1)
		nSelection = 0;

	ListBox_SetCurSel(hWndList, nSelection);
	InputDialog_SelectMapComboEntry(hWndDlg, 0);

}

static int __cdecl InputDialog_CompareControls(const void * p1, const void * p2) 
{
	ControlID * cid1 = (ControlID *)p1;
	ControlID * cid2 = (ControlID *)p2;

	return (int)lstrcmpi(cid1->szName, cid2->szName);
}

void InputDialog_FillMapCombo(HWND hWndDlg)
{
	LONG i;
	HRESULT hr;
	HWND hWndCombo;
	HWND hWndCombo2;
	LONG nIndex;

	hr = Input_GenerateAllObjects(&s_CL);
	if (FAILED(hr))
		return;

	// Sort the items
	qsort(&s_CL.controls[0],
		s_CL.controls.size(),
		sizeof(s_CL.controls[0]),
		InputDialog_CompareControls);

	hWndCombo = GetDlgItem(hWndDlg, IDC_FIRST_MAP_COMBO);
	hWndCombo2 = GetDlgItem(hWndDlg, IDC_SECOND_MAP_COMBO);

	ComboBox_ResetContent(hWndCombo);
	ComboBox_ResetContent(hWndCombo2);

	for (i = 0; i < s_CL.controls.size(); i++)
	{
		nIndex = ComboBox_InsertString(hWndCombo, -1, s_CL.controls[i].szName);

		if (nIndex != CB_ERR && nIndex != CB_ERRSPACE)
		{
			// Set data to the index of this item
			s_CL.controls[i].dwData = nIndex;

			// Set item data
			ComboBox_SetItemData(hWndCombo, nIndex, i);
		}

		nIndex = ComboBox_InsertString(hWndCombo2, -1, s_CL.controls[i].szName);

		if (nIndex != CB_ERR && nIndex != CB_ERRSPACE)
		{
			// Set data to the index of this item
			s_CL.controls[i].dwData2 = nIndex;

			// Set item data
			ComboBox_SetItemData(hWndCombo2, nIndex, i);
		}
	}


}

void InputDialog_SelectMapComboEntry(HWND hWndDlg, DWORD dwID)
{
	HWND hWndCombo;
	HWND hWndCombo2;
	DWORD dwDevice;
	DWORD dwOfsA;
	DWORD dwOfsB;
	LONG i;
	BOOL bNeedTwoCombos;
	BOOL bFirstIsAxis;

	// Check the ID is in range
	if (dwID >= NUM_INPUT_IDS)
		return;

	hWndCombo = GetDlgItem(hWndDlg, IDC_FIRST_MAP_COMBO);
	hWndCombo2 = GetDlgItem(hWndDlg, IDC_SECOND_MAP_COMBO);

	// Get the currently assigned control/offset
	dwDevice = s_Buttons[dwID].dwDevice;
	dwOfsA   = s_Buttons[dwID].dwOfsA;
	dwOfsB   = s_Buttons[dwID].dwOfsB;


	// Search through list of controls, looking for the correct index
	bFirstIsAxis = FALSE;
	for (i = 0; i < s_CL.controls.size(); i++)
	{
		if (s_CL.controls[i].dwDevice == dwDevice &&
			s_CL.controls[i].dwOfs    == dwOfsA)
		{
			// The data item is the index of the specified control
			ComboBox_SetCurSel(hWndCombo, s_CL.controls[i].dwData);

			bFirstIsAxis = s_CL.controls[i].bIsAxis;
			break;

		}
	}

	if (g_N64Buttons[dwID].bIsAxis && !bFirstIsAxis)
		bNeedTwoCombos = TRUE;
	else
		bNeedTwoCombos = FALSE;

	// Enable the second combo if this is an axis and the first control
	// is a button
	if (bNeedTwoCombos)
	{
		EnableWindow(hWndCombo2, TRUE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_SECOND_MAP_STATIC), TRUE);

		// Fill in second combo
		for (i = 0; i < s_CL.controls.size(); i++)
		{
			if (s_CL.controls[i].dwDevice == dwDevice &&
				s_CL.controls[i].dwOfs    == dwOfsB)
			{
				// The data item is the index of the specified control
				ComboBox_SetCurSel(hWndCombo2, s_CL.controls[i].dwData2);
				break;

			}
		}
	}
	else
	{
		EnableWindow(hWndCombo2, FALSE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_SECOND_MAP_STATIC), FALSE);

		// Clear second combo?
	}


	// Not found!
}

void InputDialog_AssignA(DWORD iButton, DWORD iControl)
{
	s_Buttons[iButton].dwDevice = s_CL.controls[iControl].dwDevice;
	s_Buttons[iButton].dwOfsA = s_CL.controls[iControl].dwOfs;
	s_Buttons[iButton].bIsAxis = s_CL.controls[iControl].bIsAxis;

	DBGConsole_Msg(0, "Assigning %s (%d %d %d) to %s",
		s_CL.controls[iControl].szName, s_Buttons[iButton].dwDevice,
		s_Buttons[iButton].dwOfsA, s_Buttons[iButton].bIsAxis,
		g_N64Buttons[iButton].szName);
}

void InputDialog_AssignB(DWORD iButton, DWORD iControl)
{
	s_Buttons[iButton].dwDevice = s_CL.controls[iControl].dwDevice;
	s_Buttons[iButton].dwOfsB = s_CL.controls[iControl].dwOfs;
	s_Buttons[iButton].bIsAxis = s_CL.controls[iControl].bIsAxis;

	DBGConsole_Msg(0, "Assigning %s (%d %d %d) to %s",
		s_CL.controls[iControl].szName, s_Buttons[iButton].dwDevice,
		s_Buttons[iButton].dwOfsB, s_Buttons[iButton].bIsAxis,
		g_N64Buttons[iButton].szName);
}
