/*
Copyright (C) 2001 StrmnNrmn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "stdafx.h"

//#define     D3D_OVERLOADS

#include <shlwapi.h>
#include <d3dx.h>

#include "AudioPlugin.h"


#include "unziprom.h"

#include "Dump.h"
#include "Debug.h"
#include "OpCode.h"

#include "resource.h"

#include "Memory.h"
#include "CPU.h"
#include "RDP.h"				// RDPInit/Cleanup
#include "ROM.h"				// ROM_Unload
#include "Interrupt.h"

#include "ConfigHandler.h"
#include "MainWindow.h"
//#include "DSoundHandler.h"

#include "SR.h"
#include "Patch.h"
#include "DBGConsole.h"
#include "FilenameHandler.h"

//////////////////////////////////////////////////
// Globally defined strings
TCHAR					g_szMainWindowClassname[256];

TCHAR					g_szDaedalusName[256];

TCHAR					g_szCurrentRomFileName[MAX_PATH] = "";
TCHAR					g_szCurrentAudioPluginFileName[MAX_PATH] = "";

/////////////////////////////////////////////////////////////////////
HWND					g_hMainWindow				= NULL;

HINSTANCE				g_hInstance					= NULL;
static HACCEL			g_hMainAccel				= NULL;

#ifdef DAEDALUS_LOG
HANDLE					g_hOutputLog				= INVALID_HANDLE_VALUE;
#endif


HANDLE					g_hEndBufferEvent			= NULL;


IniFile *				g_pIniFile = NULL;



// Static Declarations
int  WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst, LPSTR lpszArgs, int nWinMode);
static void LoadStrings(void);
static BOOL InitClasses();
static int  RunMain(void);


TCHAR		g_szDaedalusExeDir[MAX_PATH+1];
TCHAR		g_szDaedalusIniFile[MAX_PATH+1];

ConfigHandler *	g_pConfig;


/////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst,
				   LPSTR lpszArgs, int nWinMode)
{
	int nResult;

	g_hInstance = hThisInst;

	LoadStrings();
	GetModuleFileName(hThisInst, g_szDaedalusExeDir, MAX_PATH);
	PathRemoveFileSpec(g_szDaedalusExeDir);

	
	// Generate ini filename
	PathCombine(g_szDaedalusIniFile, g_szDaedalusExeDir, TEXT("daedalus.ini"));

	{
		ConfigHandler * pConfig = new ConfigHandler("Main");
		
		if (pConfig != NULL)
		{
			LONG nLeft, nTop, nWidth, nHeight;

			pConfig->ReadValue("UseDynaRec", (DWORD*)&g_bUseDynarec, TRUE);
			pConfig->ReadValue("ApplyPatches", (DWORD*)&g_bApplyPatches, TRUE);
			pConfig->ReadValue("TextureCRCCheck", (DWORD*)&g_bCRCCheck, TRUE);
			pConfig->ReadValue("SpeedSync", (DWORD*)&g_bSpeedSync, TRUE);
			pConfig->ReadValue("ShowDebug", (DWORD*)&g_bShowDebug, FALSE);
			pConfig->ReadString("AudioPlugin", g_szCurrentAudioPluginFileName, MAX_PATH, "");

			pConfig->ReadValue("DisplayWidth", &g_dwDisplayWidth, 640);
			pConfig->ReadValue("DisplayHeight", &g_dwDisplayHeight, 480);
			pConfig->ReadValue("DisplayFullscreen", (DWORD*)&g_bDisplayFullscreen, FALSE);

			pConfig->ReadValue("WindowLeft", (DWORD*)&nLeft, CW_USEDEFAULT);
			pConfig->ReadValue("WindowTop", (DWORD*)&nTop, CW_USEDEFAULT);
			pConfig->ReadValue("WindowWidth", (DWORD*)&nWidth, 640);
			pConfig->ReadValue("WindowHeight", (DWORD*)&nHeight, 480);

			g_rcMainWindow.left = nLeft;
			g_rcMainWindow.top = nTop;
			g_rcMainWindow.right = nLeft + nWidth;
			g_rcMainWindow.bottom = nTop + nHeight;


			switch (g_dwDisplayWidth)
			{
			case 320: case 640: case 800:
				break;
			default:
				g_dwDisplayWidth = 320;
				break;
			}

			// Hack to force correct aspect. Later we might allow non-standard res
			g_dwDisplayHeight = (g_dwDisplayWidth * 3) / 4;


			delete pConfig;
		}
	}

	g_pIniFile = new IniFile(g_szDaedalusIniFile);
	if (g_pIniFile == NULL)
	{
		MessageBox(NULL, "Unable to read IniFile from disk", g_szDaedalusName, MB_OK);
		return 1;
	}
	g_pIniFile->ReadIniFile();


	// Init memory before Console (which tries to display things that
	// are refs to memory)
	if (!Memory_Init(8*1024*1024)) return 0;

	// Don't care about failures
	DBGConsole_Enable(g_bShowDebug);


	if (!InitClasses()) return 0;
	if (FAILED(SR_Init(30000))) return 0;


	//Dump_Pifrom();

	// Open the logfile for writing
	#ifdef DAEDALUS_LOG
	{
		TCHAR szLogFileName[MAX_PATH+1];

		Dump_GetDumpDirectory(szLogFileName, TEXT(""));

		PathAppend(szLogFileName, TEXT("daedalus.txt"));

		g_hOutputLog = CreateFile(szLogFileName, // open the file 
				GENERIC_WRITE,                // open for writing 
				0,                            // do not share 
				NULL,                         // no security 
				CREATE_ALWAYS,                // replace any existing file
				FILE_ATTRIBUTE_NORMAL,        // normal file 
				NULL);                        // no attr. template 
		if (g_hOutputLog == INVALID_HANDLE_VALUE)
		{ 
			return 0;
		}
	}
	#endif

	DBGConsole_Msg(0, "[cDaedalus %s] - A Nintendo64 Emulator by StrmnNrmn", DAEDALUS_VERSION);
	DBGConsole_Msg(0, "Copyright (C) 2001 StrmnNrmn");

	DBGConsole_Msg(0, "");
	DBGConsole_Msg(0, "[WDisclaimer]");
	DBGConsole_Msg(0, "[W----------]");

	DBGConsole_Msg(0, "I do not have any association with Nintendo, or");
	DBGConsole_Msg(0, "any of its affiliates.  This program was developed for");
	DBGConsole_Msg(0, "non-commercial use and is not intended to compete ");
	DBGConsole_Msg(0, "with the Nintendo 64.");
	DBGConsole_Msg(0, "The name Nintendo and various other names and");
	DBGConsole_Msg(0, "service marks are owned by either Nintendo or their ");
	DBGConsole_Msg(0, "respective owners. ");
	DBGConsole_Msg(0, "");

	DBGConsole_Msg(0, "For more information visit [B%s]", DAEDALUS_SITE);

	DBGConsole_Msg(0, "");
	DBGConsole_Msg(0, "[WConfig]");
	DBGConsole_Msg(0, "[W------]");
	DBGConsole_Msg(0, "DynaRec: [W%s], Patches: [W%s], TextureCRC: [W%s], SpeedSync: [W%s]",
		g_bUseDynarec ? "on" : "off",
		g_bApplyPatches ? "on" : "off",
		g_bCRCCheck ? "on" : "off",
		g_bSpeedSync ? "on" : "off");

	DBGConsole_Msg(0, "");

	g_hMainWindow = Main_Create();
	if (!IsWindow(g_hMainWindow))
		return 1;
	
	ShowWindow(g_hMainWindow, nWinMode);
	UpdateWindow(g_hMainWindow);

	g_hMainAccel = LoadAccelerators(hThisInst, MAKEINTRESOURCE(IDR_APP_ACCELERATOR));

	try
	{

		nResult = RunMain();

	}
	catch (...)
	{
		MessageBox(NULL, "Exception in Daedalus", g_szDaedalusName, MB_OK);
	}


	{
		ConfigHandler * pConfig = new ConfigHandler("Main");
		
		if (pConfig != NULL)
		{
			//pConfig->WriteValue("UseDynaRec", g_bUseDynarec);
			//pConfig->WriteValue("ApplyPatches", g_bApplyPatches);
			//pConfig->WriteValue("TextureCRCCheck", g_bCRCCheck);
			//pConfig->WriteValue("SpeedSync", g_bSpeedSync);
			pConfig->WriteValue("ShowDebug", g_bShowDebug);
			pConfig->WriteString("AudioPlugin", g_szCurrentAudioPluginFileName);

			pConfig->WriteValue("DisplayWidth", g_dwDisplayWidth);
			pConfig->WriteValue("DisplayHeight", g_dwDisplayHeight);
			pConfig->WriteValue("DisplayFullscreen", g_bDisplayFullscreen);


			pConfig->WriteValue("WindowLeft", g_rcMainWindow.left);
			pConfig->WriteValue("WindowTop", g_rcMainWindow.top);
			pConfig->WriteValue("WindowWidth", g_rcMainWindow.right - g_rcMainWindow.left);
			pConfig->WriteValue("WindowHeight", g_rcMainWindow.bottom - g_rcMainWindow.top);

			delete pConfig;
		}
	}


	SR_Fini();

	ROM_Unload();

	DBGConsole_Enable(FALSE);
	Memory_Fini();

	#ifdef _debug
	CloseHandle(g_hOutputLog);
	#endif

	delete g_pIniFile;		// Will write ini file if changed
	return nResult;
}


int RunMain(void)
{

	MSG msg;
	BOOL bDone = FALSE;


	// Create DSoundNotify stuff
	g_hEndBufferEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (g_hEndBufferEvent == NULL) return 0;

	while (!bDone)
	{
		while (PeekMessage(&msg, (HWND)NULL, 0,0, PM_REMOVE)) {
			if (msg.message == WM_QUIT) {
				bDone = TRUE;
			} else {

				if (msg.hwnd == g_hMainWindow) {
					if (!TranslateAccelerator(g_hMainWindow, g_hMainAccel, &msg)) {
						TranslateMessage(&msg);
						DispatchMessage(&msg);
					}
				} else {
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
		}
	}
	// Although we may not have started the thread, try and stop it anyway
	StopCPUThread();

	return 0;
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
void
LoadStrings(void)
{
	LoadString(g_hInstance, IDS_MAINWINDOW_CLASSNAME, g_szMainWindowClassname, 256);

	LoadString(g_hInstance, IDS_DAEDALUS_STRING, g_szDaedalusName, 256);
	
}

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
BOOL
InitClasses()
{ 	
	if (!InitMainWindow())
		return FALSE;
   return TRUE; 
} 


